﻿//------------------------------------------------------------------------------
// <copyright company="Kartones.Net">
//     Copyleft (c) Kartones 2009
// </copyright> 
//------------------------------------------------------------------------------

using System;
using System.IO;
using System.Net;
using System.Text;
using System.Xml;
using System.Web;
using System.Web.Caching;

namespace KartonesNet.APIs
{
    /// <summary>
    /// Summary description for GoogleAnalytics
    /// </summary>
    public class GoogleAnalyticsAPI
    {
        #region Fields

        private string sessionToken = string.Empty;
        private string profileID = null;

        IGoogleAnalyticsDataFormatter dataFormatter = null;
        private int maxResults = 50;
        private string startingDate = string.Empty;
        private string endingDate = string.Empty;
        private bool useCache = false;
        private int cacheExpirationTime = 300;

        #endregion

        #region Properties

        /// <summary>
        /// Data formatter to use with the results of the calls
        /// </summary>
        public IGoogleAnalyticsDataFormatter DataFormatter
        {
            get
            {
                return dataFormatter;
            }
            set
            {
                if (value is IGoogleAnalyticsDataFormatter)
                {
                    dataFormatter = value;
                }
            }
        }

        /// <summary>
        /// Maximum number of results to display
        /// </summary>
        public int MaxResults
        {
            get
            {
                return maxResults;
            }
            set
            {
                if (value > 0)
                {
                    maxResults = value;
                }
            }
        }

        /// <summary>
        /// Starting date range, in format 'YYYY-MM-DD'. By default, Actual date - 1 day - 1 month
        /// </summary>
        /// <remarks>Warning: No format check is performed</remarks>
        public string StartingDate
        {
            get
            {
                return startingDate;
            }
            set
            {
                startingDate = value;
            }
        }

        /// <summary>
        /// Ending date range, in format 'YYYY-MM-DD'. By default, Actual date - 1 day
        /// </summary>
        /// <remarks>Warning: No format check is performed</remarks>
        public string EndingDate
        {
            get
            {
                return endingDate;
            }
            set
            {
                endingDate = value;
            }
        }

        /// <summary>
        /// Expiration time for the cache
        /// </summary>
        public int CacheExpirationTime
        {
            get
            {
                return cacheExpirationTime;
            }
            set
            {
                if (value > 0)
                {
                    cacheExpirationTime = value;
                    // Caching is only possible on ASP.NET
                    if (HttpContext.Current != null)
                    {
                        useCache = true;
                    }
                }
                else
                {
                    useCache = false;
                }
            }
        }

        #endregion

        #region Class Constructor

        /// <summary>
        /// Class Constructor
        /// </summary>
        public GoogleAnalyticsAPI()
        {
            DateTime date = DateTime.Now.AddDays(-1);
            startingDate = string.Format("{0:yyyy-MM-dd}", date.AddMonths(-1));
            endingDate = string.Format("{0:yyyy-MM-dd}", date);

            // Caching is only possible on ASP.NET
            if (HttpContext.Current != null)
            {
                useCache = true;
            }
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Login using ClientLogin authentication
        /// </summary>
        /// <param name="Account">Account (email)</param>
        /// <param name="Password">Password</param>
        public bool Login(string Account, string Password)
        {
            return Login(Account, Password, null);
        }

        /// <summary>
        /// Login using ClientLogin authentication and a desired Profile ID
        /// </summary>
        /// <param name="Account">Account (email)</param>
        /// <param name="Password">Password</param>
        /// <param name="DesiredProfileID">Desired profile ID to use for gathering data</param>
        /// <remarks>If the desired Profile ID is not found, it will use the first one available</remarks>
        public bool Login(string Account, string Password, string DesiredProfileID)
        {
            bool loginResult = false;

            loginResult = DoLogin(Account, Password);
            if (loginResult)
            {
                if (string.IsNullOrEmpty(DesiredProfileID))
                {
                    profileID = GetProfileID(null);
                }
                else
                {
                    profileID = GetProfileID(DesiredProfileID);
                }

                if (profileID == string.Empty)
                {
                    loginResult = false;
                }
            }

            return loginResult;
        }

        /// <summary>
        /// Performs a Google Analytics call for data
        /// </summary>
        /// <param name="Dimensions">Array specifying the dimensions to query</param>
        /// <param name="Metrics">Array specifying the metrics to query</param>
        /// <param name="OrderBy">Metric to order by</param>
        /// <returns>Formatted data</returns>
        /// <exception cref="NullReferenceException">DataFormatter not specified</exception>
        /// <exception cref="EmptyQueryResponseGoogleAnalyticsException">Query returned no data</exception>
        public string GADataCall(string[] Dimensions, string[] Metrics, string OrderBy)
        {
            if (dataFormatter == null)
            {
                throw new NullReferenceException("DataFormatter not specified");
            }
            if (Dimensions.Length > 7)
            {
                throw new IndexOutOfRangeException("Maximum allowed dimensions is 7");
            }
            if (Metrics.Length > 10)
            {
                throw new IndexOutOfRangeException("Maximum allowed metrics is 10");
            }

            // Prepare the query
            StringBuilder gaQuery = new StringBuilder();

            gaQuery.Append("https://www.google.com/analytics/feeds/data?ids=ga:");
            gaQuery.Append(profileID);

            if (Dimensions.Length > 0)
            {
                gaQuery.Append("&dimensions=");
                for (int index = 0; index < Dimensions.Length; index++)
                {
                    gaQuery.Append(Dimensions[index]);
                    if (Dimensions.Length > 1 && index < (Dimensions.Length - 1))
                    {
                        gaQuery.Append(",");
                    }
                }
            }

            gaQuery.Append("&metrics=");
            for (int index = 0; index < Metrics.Length; index++)
            {
                gaQuery.Append(Metrics[index]);
                if (Metrics.Length > 1 && index < (Metrics.Length - 1))
                {
                    gaQuery.Append(",");
                }
            }

            if (!string.IsNullOrEmpty(OrderBy))
            {
                gaQuery.Append("&sort=");
                gaQuery.Append(OrderBy);
            }
            gaQuery.Append("&start-date=");
            gaQuery.Append(startingDate);
            gaQuery.Append("&end-date=");
            gaQuery.Append(endingDate);
            gaQuery.Append("&max-results=");
            gaQuery.Append(maxResults);
            
            // Perform the request
            string gaData = CacheableHTTPCall(gaQuery.ToString());

            if (gaData == string.Empty)
            {
                throw new EmptyQueryResponseGoogleAnalyticsException(gaQuery.ToString());
            }

            // Convert response to XML
            XmlDocument resultsXML = new XmlDocument();
            resultsXML.LoadXml(gaData);
            XmlNodeList entries = resultsXML.GetElementsByTagName("entry");

            // Format the results
            StringBuilder results = new StringBuilder();
            string fieldValue = string.Empty;

            for (int index = 0; index < entries.Count; index++)
            {
                results.Append(dataFormatter.PreElementTag());

                for (int dimensionIndex = 0; dimensionIndex < Dimensions.Length; dimensionIndex++)
                {
                    results.Append(dataFormatter.PreFieldTag());
                    fieldValue = entries[index].ChildNodes[4 + dimensionIndex].Attributes["value"].Value;
                    results.Append(fieldValue);
                    results.Append(dataFormatter.PostFieldTag());
                }

                for (int metricIndex = 0; metricIndex < Metrics.Length; metricIndex++)
                {
                    results.Append(dataFormatter.PreFieldTag());
                    fieldValue = entries[index].ChildNodes[4 + Dimensions.Length + metricIndex].Attributes["value"].Value;
                    results.Append(fieldValue);
                    results.Append(dataFormatter.PostFieldTag());
                }

                results.Append(dataFormatter.PostElementTag());
            }

            return results.ToString();

        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Performs the actual login to GA API
        /// </summary>
        /// <param name="Account">Account (email)</param>
        /// <param name="Password">Password</param>
        /// <returns>Result of the Login operation</returns>
        private bool DoLogin(string Account, string Password)
        {
            StringBuilder postData = new StringBuilder(150);
            postData.Append("Email=");
            postData.Append(Account);
            postData.Append("&Passwd=");
            postData.Append(Password);
            // GOOGLE
            postData.Append("&accountType=HOSTED_OR_GOOGLE&service=analytics&source=Kartones.Net-GAStats-1.01");

            ASCIIEncoding encoding = new ASCIIEncoding();
            byte[] data = encoding.GetBytes(postData.ToString());

            HttpWebRequest request = null;
            try
            {
                request = (HttpWebRequest)WebRequest.Create("https://www.google.com/accounts/ClientLogin");
                request.Method = "POST";
                request.ContentType = "application/x-www-form-urlencoded";
                request.ContentLength = data.Length;
                Stream requestStream = request.GetRequestStream();
                requestStream.Write(data, 0, data.Length);
                requestStream.Close();
            }
            catch (WebException)
            {
                return false;
            }

            string responseData = string.Empty;
            try
            {
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                StreamReader streamReader = new StreamReader(response.GetResponseStream(), Encoding.GetEncoding("utf-8"));
                responseData = streamReader.ReadToEnd();
            }
            catch (WebException)
            {
                return false;
            }

            string[] auth = responseData.Split(new string[] { "Auth=" }, StringSplitOptions.None);

            // Store the authorization token
            sessionToken = auth[1];

            return true;
        }

        /// <summary>
        /// Gets the profile ID, either the one we wanted, or the first one available
        /// </summary>
        /// <param name="DesiredProfileID">Desired profile ID</param>
        /// <returns>string containing the ProfileID</returns>
        private string GetProfileID(string DesiredProfileID)
        {
            string profile = string.Empty;

            if (!string.IsNullOrEmpty(DesiredProfileID))
            {
                profile = DesiredProfileID;
            }
            else
            {
                HttpWebRequest request = null;
                try
                {
                    request = (HttpWebRequest)WebRequest.Create("https://www.google.com/analytics/feeds/accounts/default");
                    request.Headers.Add("Authorization: GoogleLogin auth=" + sessionToken);
                }
                catch (WebException)
                {
                    return string.Empty;
                }

                string responseData = string.Empty;
                try
                {
                    HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                    StreamReader streamReader = new StreamReader(response.GetResponseStream(), Encoding.GetEncoding("utf-8"));
                    responseData = streamReader.ReadToEnd();
                }
                catch (WebException)
                {
                    return string.Empty;
                }

                // Convert response to XML
                XmlDocument accountinfoXML = new XmlDocument();
                accountinfoXML.LoadXml(responseData);

                XmlNodeList entries = accountinfoXML.GetElementsByTagName("entry");

                XmlNamespaceManager namespaceMgr = new XmlNamespaceManager(accountinfoXML.NameTable);
                namespaceMgr.AddNamespace("dxp", "http://schemas.google.com/analytics/2009");

                profile = entries.Item(0).SelectSingleNode("dxp:property[@name='ga:profileId']", namespaceMgr).Attributes["value"].Value;
            }

            return profile;
        }

        /// <summary>
        /// Performs a HttpRequest-HttpResponse call, if no cached value exists for it
        /// </summary>
        /// <param name="URL">string URL to call</param>
        /// <returns>string results returned</returns>
        /// <exception cref="WebRequestGoogleAnalyticsException">Error performing Web Request</exception>
        private string CacheableHTTPCall(string URL)
        {
            bool foundCachedValues = false;
            string results = string.Empty;

            // Search in the cache
            if (useCache)
            {
                results = (string)HttpContext.Current.Cache.Get(URL);
                if (!string.IsNullOrEmpty(results))
                {
                    foundCachedValues = true;
                }
            }

            if (!foundCachedValues)
            {
                HttpWebRequest request = null;
                try
                {
                    request = (HttpWebRequest)WebRequest.Create(URL);
                    request.Headers.Add("Authorization: GoogleLogin auth=" + sessionToken);
                }
                catch (WebException ex)
                {
                    throw new WebRequestGoogleAnalyticsException(
                        string.Format("Error '{0}' creating Http Request '{1}'",
                                        ex.Message, URL));
                }

                try
                {
                    HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                    StreamReader streamReader = new StreamReader(response.GetResponseStream(), Encoding.GetEncoding("utf-8"));
                    results = streamReader.ReadToEnd();
                }
                catch (WebException ex)
                {
                    throw new WebRequestGoogleAnalyticsException(
                       string.Format("Error '{0}' executing Http Request '{1}' session token '{2}'",
                                        ex.Message, URL, sessionToken));
                }

                // Cache results if possible
                if (useCache && results != string.Empty)
                {
                    HttpContext.Current.Cache.Insert(
                        URL, results, null, DateTime.UtcNow.AddSeconds(cacheExpirationTime), Cache.NoSlidingExpiration);
                }
            }

            return results;
        }

        #endregion
    }

    /// <summary>
    /// Exception to throw if a GA query returns empty
    /// </summary>
    public class WebRequestGoogleAnalyticsException : ApplicationException
    {
        /// <summary>
        /// Class constructor
        /// </summary>
        public WebRequestGoogleAnalyticsException(string Message) : base(Message)
        {
        }
    }

    /// <summary>
    /// Exception to throw if a GA query returns empty
    /// </summary>
    public class EmptyQueryResponseGoogleAnalyticsException : ApplicationException
    {
        /// <summary>
        /// Class constructor
        /// </summary>
        public EmptyQueryResponseGoogleAnalyticsException(string Query) : 
            base("Google Analytics returned no data.Query: " + Query)
        {
        }
    }
}