﻿using System;
using System.Text;
using System.Xml;
using System.Net;
using System.IO;

namespace KartonesNet.APIs
{
    /// <summary>
    /// Allows to retrieve Feedburner's Awareness API info for a given feed if available
    /// </summary>
    public class GoogleFeedburnerAwareness
    {
        #region Fields

        private const string feedburnerAPIURL = 
            "https://feedburner.google.com/api/awareness/1.0/GetFeedData?uri={0}&dates={1},{2}";
        private string feedURI;
        private int monthsPastToRetrieve;
        private XmlDocument responseXML;
        private string statsEndingDate;
        private string statsStartingDate;

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the feed URI
        /// </summary>
        public string FeedURI
        {
            get
            {
                return this.feedURI;
            }
            set
            {
                if (!string.IsNullOrEmpty(value))
                {
                    this.feedURI = value;
                }
            }
        }

        /// <summary>
        /// Gets or sets the number of past months to retrieve.
        /// By default retrieves past 6 months.
        /// </summary>
        public int MonthsPastToRetrieve
        {
            get
            {
                return this.monthsPastToRetrieve;
            }
            set
            {
                if (value > 0)
                {
                    this.monthsPastToRetrieve = value;
                }
            }
        }

        #endregion

        #region Class Constructor

        /// <summary>
        /// Class constructor
        /// </summary>
        /// <param name="FeedURI">URI of the feed from which to get the info of</param>
        public GoogleFeedburnerAwareness(string FeedURI) : this(FeedURI, 6)
        {
        }

        /// <summary>
        /// Class constructor
        /// </summary>
        /// <param name="FeedURI">URI of the feed from which to get the info of</param>
        /// <param name="MonthsPastToRetrieve">Number of months to retrieve info of</param>
        public GoogleFeedburnerAwareness(string FeedURI, int MonthsPastToRetrieve)
        {
            this.feedURI = string.Empty;
            this.monthsPastToRetrieve = 6;
            this.statsStartingDate = string.Empty;
            this.statsEndingDate = string.Empty;
            this.MonthsPastToRetrieve = MonthsPastToRetrieve;
            this.FeedURI = FeedURI;
            this.statsStartingDate = string.Format("{0}-{1}-{2}", 
                                                    DateTime.Now.AddMonths(-this.monthsPastToRetrieve).Year, 
                                                    DateTime.Now.AddMonths(-this.monthsPastToRetrieve).Month, 
                                                    DateTime.Now.AddMonths(-this.monthsPastToRetrieve).Day);
            this.statsEndingDate = string.Format("{0}-{1}-{2}", 
                                                    DateTime.Now.Year, 
                                                    DateTime.Now.Month, 
                                                    DateTime.Now.Day);
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Fetches the feed in the class
        /// </summary>
        /// <returns>true if operation succeeded, false otherwise</returns>
        public bool FetchFeed()
        {
            string feedContent;
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(
                string.Format("https://feedburner.google.com/api/awareness/1.0/GetFeedData?uri={0}&dates={1},{2}", 
                                this.feedURI, this.statsStartingDate, this.statsEndingDate));
            request.Method = "GET";
            try
            {
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.UTF8);
                feedContent = reader.ReadToEnd();
                reader.Close();
                response.Close();
            }
            catch (Exception)
            {
                feedContent = string.Empty;
            }

            if (!string.IsNullOrEmpty(feedContent))
            {
                try
                {
                    // Store the data in the XMLDocument
                    this.responseXML = new XmlDocument();
                    this.responseXML.Load(new StringReader(feedContent));
                    // Check for the success API message
                    return (this.responseXML.SelectSingleNode("/rsp").Attributes.GetNamedItem("stat").Value == "ok");
                }
                catch (Exception)
                {
                    this.responseXML = null;
                }
            }
            return false;
        }

        /// <summary>
        /// Parses and gets all the entries
        /// </summary>
        /// <returns>GoogleFeedburnerEntryList containing all the entries</returns>
        public GoogleFeedburnerEntryList GetEntries()
        {
            GoogleFeedburnerEntryList list = new GoogleFeedburnerEntryList();
            list.MinCirculation = long.MinValue;
            list.MaxCirculation = 0;
            list.MinHits = long.MinValue;
            list.MaxHits = 0;
            list.MinReach = long.MinValue;
            list.MaxReach = 0;
            list.Entries = new GoogleFeedburnerEntry[0];

            if (this.responseXML != null)
            {
                XmlNodeList entries = this.responseXML.SelectNodes("/rsp/feed/entry");

                list.Entries = new GoogleFeedburnerEntry[entries.Count];
                for (int index = 0; index < entries.Count; index++)
                {
                    list.Entries[index].Date = 
                        DateTime.Parse(entries[index].Attributes.GetNamedItem("date").Value);

                    list.Entries[index].Circulation = 
                        long.Parse(entries[index].Attributes.GetNamedItem("circulation").Value);
                    if (list.Entries[index].Circulation > list.MaxCirculation)
                    {
                        list.MaxCirculation = list.Entries[index].Circulation;
                    }
                    if (list.Entries[index].Circulation < list.MinCirculation)
                    {
                        list.MinCirculation = list.Entries[index].Circulation;
                    }
                    
                    list.Entries[index].Hits = 
                        long.Parse(entries[index].Attributes.GetNamedItem("hits").Value);
                    if (list.Entries[index].Hits > list.MaxHits)
                    {
                        list.MaxHits = list.Entries[index].Hits;
                    }
                    if (list.Entries[index].Hits < list.MinHits)
                    {
                        list.MinHits = list.Entries[index].Hits;
                    }
                    
                    list.Entries[index].Reach = 
                        long.Parse(entries[index].Attributes.GetNamedItem("reach").Value);
                    if (list.Entries[index].Reach > list.MaxReach)
                    {
                        list.MaxReach = list.Entries[index].Reach;
                    }
                    if (list.Entries[index].Reach < list.MinReach)
                    {
                        list.MinReach = list.Entries[index].Reach;
                    }
                }
            }
            return list;
        }

        #endregion
    }
}
