//------------------------------------------------------------------------------
// <copyright company="Kartones.Net">
//     Copyleft (c) Kartones 2009
// </copyright> 
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using CommunityServer.Blogs.Components;
using CommunityServer.Blogs.Controls;
using CommunityServer.Components;
using CommunityServer.Controls;

namespace KartonesNet.Components
{
    /// <summary>
    /// Blog post list control that returns all posts from a blog
    /// </summary>
    public class AllWeblogPostsList : PreTemplatedListBase
    {
        #region Fields

        private bool isImplicitDataSource = true;
        private List<WeblogPost> blogPosts;
        private bool sortByDateDesc = false;
        private bool skipEmptyCategories = false;

        #endregion

        #region Properties

        public bool SortByDateDesc
        {
            get
            {
                return sortByDateDesc;
            }
            set
            {
                sortByDateDesc = value;
            }
        }


        /// <summary>
        /// If true, only posts with at least one category/tag will be listed
        /// </summary>
        public bool SkipEmptyCategories
        {
            get
            {
                return skipEmptyCategories;
            }
            set
            {
                skipEmptyCategories = value;
            }
        }

        /// <summary>
        /// Datasource containing the posts
        /// </summary>
        public override object DataSource
        {
            get
            {
                // Only retrieve them if not present already
                if (blogPosts == null)
                {
                    BlogThreadQuery threadQuery;
                    CSContext csContext = WeblogControlUtility.Instance().GetCurrentCSContext(this.Page);

                    Weblog blog = WeblogControlUtility.Instance().GetCurrentWeblog(this);
                    if (blog == null)
                    {
                        EventLogs.Warn("Could not retrieve blog", "AllWeblogPostsList", 985);
                        return new List<WeblogPost>();
                    }

                    threadQuery = BlogThreadQuery.CreateNewSingleBlogQuery(blog.SectionID);
                    // Get all posts
                    threadQuery.PageIndex = 0;
                    threadQuery.PageSize = blog.PostCount;

                    threadQuery.IncludeCategories = true;
                    threadQuery.BlogPostType = BlogPostType.Post;

                    if (sortByDateDesc)
                    {
                        threadQuery.SortBy = BlogThreadSortBy.MostRecent;
                    }

                    ThreadSet threads = WeblogPosts.GetBlogThreads(threadQuery, true, false);
                    blogPosts = new List<WeblogPost>();

                    foreach (WeblogPost post in threads.Threads)
                    {
                        if (!skipEmptyCategories || post.Categories.Length > 0)
                        {
                            blogPosts.Add(post);
                        }
                    }

                    if (!sortByDateDesc)
                    {
                        // Anonymous delegate-based alphabetical sorting
                        blogPosts.Sort(
                            delegate(WeblogPost post1, WeblogPost post2)
                            {
                                return string.Compare(post1.Subject, post2.Subject, true);
                            }
                            );
                    }
                }

                return blogPosts;
            }
            set
            {
                if (value is List<WeblogPost>)
                {
                    blogPosts = value as List<WeblogPost>;
                    isImplicitDataSource = false;
                    RequiresDataBinding = true;
                    OnDataPropertyChanged();
                }
                else if (value == null)
                {
                    blogPosts = null;
                    isImplicitDataSource = true;
                    OnDataPropertyChanged();
                    RequiresDataBinding = true;
                }
                else
                    throw new InvalidCastException("DataSource must be a valid List<WeblogPost> object");
            }
        }

        #endregion

        #region Protected Methods

        /// <summary>
        /// AddPropertyControls
        /// </summary>
        /// <param name="dataBinding">dataBinding</param>
        protected override void AddPropertyControls(bool dataBinding)
        {
            base.AddPropertyControls(dataBinding);
        }

        /// <summary>
        /// Triggers when page index changes
        /// </summary>
        /// <param name="sender">sender</param>
        /// <param name="e">PagerEventArgs</param>
        protected void PageIndexChanged(IPager sender, PagerEventArgs e)
        {
            if (isImplicitDataSource)
            {
                FlagForDataBinding();
            }
        }

        /// <summary>
        /// Binds the default content for an item
        /// </summary>
        /// <param name="listItem"></param>
        /// <param name="dataItemContainer"></param>
        protected override void BindDefaultItemContent(PreTemplatedListBase.ListItemTemplateListItem listItem, 
                                                        System.Web.UI.IDataItemContainer dataItemContainer)
        {
            WeblogPost item = dataItemContainer.DataItem as WeblogPost;
            if (item != null)
            {
                WeblogPostData postData = new WeblogPostData();

                postData.DataSource = item;
                postData.Text = item.Subject;
                postData.LinkTo = WeblogPostLinkTo.Post;

                listItem.Controls.Add(postData);
            }
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Flags for databinding
        /// </summary>
        public override void FlagForDataBinding()
        {
            this.DataSource = null;
            base.FlagForDataBinding();
        }

        #endregion
    }

    
    
}
