using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Caching;
using System.Web.UI;
using CommunityServer.Blogs.Components;
using CommunityServer.Blogs.Controls;
using CommunityServer.Components;
using CommunityServer.Controls;

namespace KartonesNet.Components
{
    /// <summary>
    /// Available criterias by which to obtain the related posts
    /// </summary>
    enum SearchCriterias
    {
        /// <summary>
        /// Get the X most recent ones
        /// </summary>
        ByMostRecent = 0,
        /// <summary>
        /// Get the X most viewed ones
        /// </summary>
        ByMostViews = 1
    }

    /// <summary>
    /// This webcontrol renders a list of X related posts based on similar tags to the ones
    /// of the context post
    /// </summary>
    public class RelatedPosts : WrappedContentBase
    {
        #region Fields

        /// <summary>
        /// This value is constant because almost all the sites/blogs show 5 related posts
        /// </summary>
        private const int numRelatedPosts = 5;

        private SearchCriterias searchCriteria = SearchCriterias.ByMostRecent;
        private bool increaseMultipleTagCoincidenceWeight = false;
        private string relatedPostsLiteral = "Related posts:";
        private bool useCaching = true;

        #endregion

        #region Properties

        /// <summary>
        /// Sets the related posts search criteria
        /// </summary>
        /// <remarks>By default is by most recent</remarks>
        public string SearchCriteria
        {
            set
            {
                switch (value.ToLower())
                {
                    case "date":
                        searchCriteria = SearchCriterias.ByMostRecent;
                        break;
                    case "views":
                        searchCriteria = SearchCriterias.ByMostViews;
                        break;
                }
            }
        }

        /// <summary>
        /// Literal to append before the related post list.
        /// </summary>
        /// <remarks>By default "Related posts:"</remarks>
        public string RelatedPostsLiteral
        {
            set
            {
                if (!string.IsNullOrEmpty(value))
                {
                    relatedPostsLiteral = value;
                }
            }
        }

        /// <summary>
        /// If true, posts that have at least two shared tags with the original post
        /// will have heavier weight for the selection process
        /// </summary>
        /// <remarks>false by default</remarks>
        public bool IncreaseMultipleTagCoincidenceWeight
        {
            set
            {
                increaseMultipleTagCoincidenceWeight = value;
            }
        }

        /// <summary>
        /// Activates or deactivates caching of results
        /// </summary>
        public bool UseCaching
        {
            set
            {
                useCaching = value;
            }
        }

        #endregion

        #region Protected Methods

        /// <summary>
        /// Creates and binds the controls that represent the tag list
        /// </summary>
        /// <param name="ParentControl">Parent control that will contain the tag list</param>
        /// <param name="DataItemContainer">DataItemContainer</param>
        /// <exception cref="NullReferenceException">If could not obtain the post</exception>
        protected override void BindDefaultContent(Control ParentControl, IDataItemContainer DataItemContainer)
        {
            BlogThreadQuery threadQuery;
            ThreadSet threads;
            bool postExists;
            int index;
            StringBuilder relatedPostsMarkup;

            List<WeblogPost> relatedPostsFound = new List<WeblogPost>();
            // Duplicates have more weight as they share more than one tag in common
            List<WeblogPost> duplicatedPostsFound = new List<WeblogPost>();
            List<WeblogPost> finalList;

            WeblogPost post = WeblogControlUtility.Instance().GetCurrentWeblogPost(this);
            if (post == null)
                throw new NullReferenceException("post null");

            string cachedOutput = null;
            if (useCaching)
            {
                cachedOutput = CSCache.Get("RELATEDPOSTS_" + post.PostID.ToString()) as string;
            }
            if (cachedOutput == null || !useCaching)
            {
                foreach (string tag in post.Categories)
                {
                    threadQuery = BlogThreadQuery.CreateNewSingleBlogQuery(post.Weblog.SectionID);
                    threadQuery.PageIndex = 0;
                    threadQuery.PageSize = numRelatedPosts;
                    threadQuery.IncludeCategories = false;
                    threadQuery.BlogPostType = BlogPostType.Post;
                    switch (searchCriteria)
                    {
                        case SearchCriterias.ByMostRecent:
                            threadQuery.SortBy = BlogThreadSortBy.MostRecent;
                            break;
                        case SearchCriterias.ByMostViews:
                            threadQuery.SortBy = BlogThreadSortBy.MostViewed;
                            break;
                    }

                    // Each query, only with one tag
                    threadQuery.Tags = new string[] { tag };

                    threads = WeblogPosts.GetBlogThreads(threadQuery, false, false);

                    foreach (WeblogPost foundPost in threads.Threads)
                    {
                        if (foundPost.PostID != post.PostID)
                        {
                            // If not already present on the list, add it
                            postExists = false;
                            index = 0;
                            while (index < relatedPostsFound.Count && !postExists)
                            {
                                if (relatedPostsFound[index].PostID == foundPost.PostID)
                                {
                                    postExists = true;
                                }
                                else
                                {
                                    index++;
                                }
                            }
                            if (!postExists)
                            {
                                relatedPostsFound.Add(foundPost);
                            }
                            else
                            {
                                // If present, repeat the process with duplicated posts list
                                // but only if we are going to use that list                                
                                if (increaseMultipleTagCoincidenceWeight)
                                {
                                    postExists = false;
                                    index = 0;
                                    while (index < duplicatedPostsFound.Count && !postExists)
                                    {
                                        if (duplicatedPostsFound[index].PostID == foundPost.PostID)
                                        {
                                            postExists = true;
                                        }
                                        else
                                        {
                                            index++;
                                        }
                                    }
                                    if (!postExists)
                                    {
                                        duplicatedPostsFound.Add(foundPost);
                                    }
                                }
                            }
                        }
                    }
                }

                SortAndTruncateList(relatedPostsFound, numRelatedPosts);
                if (increaseMultipleTagCoincidenceWeight)
                    finalList = IntersectLists(duplicatedPostsFound, relatedPostsFound, numRelatedPosts);
                else
                    finalList = relatedPostsFound;

                relatedPostsMarkup = new StringBuilder();
                relatedPostsMarkup.Append("<br /><div>");
                relatedPostsMarkup.Append(relatedPostsLiteral);
                relatedPostsMarkup.Append("<ul>");
                for (int postIndex = 0; postIndex < finalList.Count; postIndex++)
                {
                    relatedPostsMarkup.Append("<li><a href='");
                    relatedPostsMarkup.Append(finalList[postIndex].ViewPostURL);
                    relatedPostsMarkup.Append("'>");
                    relatedPostsMarkup.Append(finalList[postIndex].Subject);
                    relatedPostsMarkup.Append("</a></li>");
                }
                relatedPostsMarkup.Append("</ul></div>");

                if (useCaching)
                {
                    // Insert into cache
                    CSCache.Insert("RELATEDPOSTS_" + post.PostID.ToString(), relatedPostsMarkup.ToString(),
                        1800, CacheItemPriority.Normal);
                }

                ParentControl.Controls.Add(new LiteralControl(relatedPostsMarkup.ToString()));
            }
            else
            {
                ParentControl.Controls.Add(new LiteralControl(cachedOutput));
            }
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Sorts and then truncates a given list
        /// </summary>
        /// <param name="PostList">List of WeblogPost elements to sort by the specified SearchCriteria</param>
        /// <param name="Length">Number of elements maximum to keep (the rest will be removed)</param>
        /// <remarks>As PostList is passed by reference, it will be directly modified</remarks>
        private void SortAndTruncateList(List<WeblogPost> PostList, int Length)
        {
            // Sort the list of found posts by visits
            if (PostList.Count > 1)
            {
                switch (searchCriteria)
                {
                        // Different sorting depending on the criteria
                    case SearchCriterias.ByMostRecent:
                        PostList.Sort(
                            delegate(WeblogPost post1, WeblogPost post2)
                            {
                                return post2.PostDate.CompareTo(post1.PostDate);
                            }
                            );
                        break;
                    case SearchCriterias.ByMostViews:
                        PostList.Sort(
                            delegate(WeblogPost post1, WeblogPost post2)
                            {
                                return (post2.Views + post2.AggViews).CompareTo(post1.Views + post1.AggViews);
                            }
                            );
                        break;
                }
            }

            // Keep only the top X sorted posts
            if (PostList.Count > Length)
            {
                PostList.RemoveRange(Length, PostList.Count - Length);
                PostList.TrimExcess();
            }
        }

        /// <summary>
        /// Intersects two lists, filling first with the 'high priority' posts and then with the normal ones.
        /// </summary>
        /// <param name="HighPriorityPosts">High priority posts</param>
        /// <param name="NormalPosts">Normal priority posts</param>
        /// <param name="Length">Maximum number of posts that the resulting list can contain</param>
        /// <returns>Intersected and truncated post list</returns>
        /// <remarks></remarks>
        private List<WeblogPost> IntersectLists(List<WeblogPost> HighPriorityPosts, List<WeblogPost> NormalPosts, int Length)
        {
            List<WeblogPost> finalList = new List<WeblogPost>(Length);

            if (HighPriorityPosts.Count > Length)
            {
                finalList.AddRange(HighPriorityPosts.GetRange(0, Length));
            }
            else
            {
                finalList.AddRange(HighPriorityPosts.GetRange(0, HighPriorityPosts.Count));
                if (finalList.Count < Length)
                {
                    // Avoid out of index exceptions, we might have less than expected posts!
                    if (Length - finalList.Count <= NormalPosts.Count)
                    {
                        finalList.AddRange(NormalPosts.GetRange(0, Length - finalList.Count));
                    }
                    else
                    {
                        finalList.AddRange(NormalPosts.GetRange(0, NormalPosts.Count));
                    }
                }
            }

            HighPriorityPosts.Clear();
            NormalPosts.Clear();

            return finalList;
        }

        #endregion
    }
}
