using System.Text;
using System.Web.UI;
using CommunityServer.Blogs.Components;
using CommunityServer.Blogs.Controls;
using CommunityServer.Components;
using CommunityServer.Controls;
using System.Collections.Generic;
using System;

namespace KartonesNet.Components
{
    /// <summary>
    /// Control providing a readonly WeblogPost visual (image-based) tag list.
    /// </summary>
    public class WeblogPostVisualTagList : WrappedContentBase
    {
        #region Fields

        private string tagImagesURL = string.Empty;
        private string imagesExtension = "png";
        private string tagListPrefixLiteral = string.Empty;
        private Dictionary<string,string> tagAliases = new Dictionary<string,string>(StringComparer.OrdinalIgnoreCase);

        #endregion

        #region Properties

        /// <summary>
        /// URL of the folder containing the .gif images representing the tags
        /// </summary>
        public string TagImagesURL
        {
            set
            {
                tagImagesURL = value;
            }
        }

        /// <summary>
        /// Extension for the images
        /// </summary>
        public string ImagesExtension
        {
            set
            {
                imagesExtension = "." + value.ToLower();
            }
        }

        /// <summary>
        /// Literal to append before the tag list
        /// </summary>
        public string TagListPrefixLiteral
        {
            set
            {
                tagListPrefixLiteral = value;
            }
        }

        /// <summary>
        /// Sets the list of tag aliases, with the following format:
        /// alias1[,alias2,...]=tag1;alias3=tag3;...
        /// </summary>
        /// <remarks>There are no duplicate checks, warning!</remarks>
        public string TagAliases
        {
            set
            {
                // TODO: Cache this, as it will probably not change in time...
                if (!string.IsNullOrEmpty(value))
                {
                    string[] elements = value.Split(';');
                    string[] elementParts;
                    string[] aliases;

                    for (int index = 0; index < elements.Length; index++)
                    {
                        elementParts = elements[index].Split('=');
                        // tag to substitute for its contained on elementParts[1]
                        aliases = elementParts[0].Split(',');
                        for (int index2 = 0; index2 < aliases.Length; index2++)
                        {
                            tagAliases.Add(aliases[index2], elementParts[1]);
                        }
                    }
                }
            }
        }

        #endregion

        #region Protected Methods

        /// <summary>
        /// Creates and binds the controls that represent the tag list
        /// </summary>
        /// <param name="ParentControl">Parent control that will contain the tag list</param>
        /// <param name="DataItemContainer">DataItemContainer</param>
        protected override void BindDefaultContent(Control ParentControl, IDataItemContainer DataItemContainer)
        {
            WeblogPost post = WeblogControlUtility.Instance().GetCurrentWeblogPost(this);
            string blogApplicationKey = WeblogControlUtility.Instance().GetCurrentWeblog(this).ApplicationKey;

            if (post != null && !string.IsNullOrEmpty(blogApplicationKey))
            {
                string[] tags = GetTags(post);
                string formattedTags = FormatTags(tags, blogApplicationKey);
                ParentControl.Controls.Add(new LiteralControl(formattedTags));
            }
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Gets all the tags of a post
        /// </summary>
        /// <param name="BlogPost">Post</param>
        /// <returns>list of tags</returns>
        private string[] GetTags(Post BlogPost)
        {
            string[] results;

            if (BlogPost == null)
            {
                results = new string[0];
            }

            if (BlogPost.Categories != null)
            {
                //results = BlogPost.Categories;

                List<string> aliases = new List<string>();
                for (int index = 0; index < BlogPost.Categories.Length; index++)
                {
                    if (!BlogPost.Categories[index].StartsWith("["))
                    {
                        aliases.Add(BlogPost.Categories[index]);
                    }
                }
                results = aliases.ToArray();

            }
            else
            {
                results = new string[0];
            }

            return results;
        }

        /// <summary>
        /// Formats the tag list, creating hyperlinks to browse posts with same tag
        /// </summary>
        /// <param name="TagList">List of tags</param>
        /// <param name="BlogApplicationKey">Blog application key</param>
        /// <returns>HTML markup ready to display</returns>
        private string FormatTags(string[] TagList, string BlogApplicationKey)
        {
            StringBuilder tagsMarkup = new StringBuilder();

            if (TagList != null && TagList.Length > 0)
            {
                tagsMarkup.Append(tagListPrefixLiteral);
                string currTag = string.Empty;
                for (int index = 0; index < TagList.Length; index++)
                {
                    // Check if it is an alias and replace if so
                    currTag = tagAliases.ContainsKey(TagList[index]) ? tagAliases[TagList[index]] : TagList[index];

                    if (index > 0)
                    {
                        // tag delimiter
                        tagsMarkup.Append(" ");
                    }
                    tagsMarkup.Append("<a href='");
                    tagsMarkup.Append(BlogUrls.Instance().TagsBrowser(BlogApplicationKey, new string[] { TagList[index] }));
                    tagsMarkup.Append("' rel='tag'><img src='");
                    tagsMarkup.Append(tagImagesURL);
                    tagsMarkup.Append(currTag.ToLower().Replace(' ', '_'));
                    tagsMarkup.Append(imagesExtension);
                    tagsMarkup.Append("' alt='");
                    tagsMarkup.Append(currTag);
                    tagsMarkup.Append("' title='");
                    tagsMarkup.Append(currTag);
                    tagsMarkup.Append("' /></a>");
                }
            }

            return tagsMarkup.ToString();
        }

        #endregion
    }
}
