//------------------------------------------------------------------------------
// <copyright company="Kartones.Net">
//     Copyleft (c) Kartones 2009
// </copyright> 
//------------------------------------------------------------------------------

using System;
using System.Collections;
using CommunityServer.Components;
using CommunityServer.Spam;
using CommunityServer.Spam.Rules;

namespace KartonesNet.Modules.Rules
{
	/// <summary>
	/// Rates spam factor based on the post length.
	/// </summary>
	public class PostLengthSpamRule : SpamRule
    {
        #region Fields

        private static readonly Guid ruleID = new Guid("8D45AE31-C317-4790-9817-85F756ED3458");
		private static readonly int defaultPoints = 5;
        private static readonly int defaultThreshold = 10;

        #endregion

        #region Properties

        /// <summary>
        /// Name of the Spam Rule
        /// </summary>
        public override string Name
        {
            get
            {
                return "Post Length";
            }
        }

        /// <summary>
        /// Description of the Spam rule
        /// </summary>
        public override string Description
        {
            get
            {
                return "Rates spam factor based on the length of the post";
            }
        }

        /// <summary>
        /// GUID of the Spam Rule
        /// </summary>
        public override Guid RuleID
        {
            get
            {
                return ruleID;
            }
        }

        #endregion

        #region Class Constructor

        /// <summary>
        /// Class Constructor
        /// </summary>
        public PostLengthSpamRule()
		{
        }

        #endregion

        #region SpamRule Methods

        /// <summary>
        /// Gets the Spam Rule settings
        /// </summary>
        /// <returns>Arraylist with the settings</returns>
        public override ArrayList GetAvailableSettings()
		{
			ArrayList settings = new ArrayList();
			settings.Add(new RuleSetting(ruleID, "points", 
                "Points to assign if threshold not achieved. Adds 1 additional point per character behind threshold.", 
                defaultPoints.ToString()));
			settings.Add(new RuleSetting(ruleID, "threshold", "Threshold/minimum post length required", defaultThreshold.ToString()));
			return settings;
		}

        /// <summary>
        /// Rates a post/comment
        /// </summary>
        /// <param name="Post">Post to rate</param>
        /// <param name="Args">CSPostEventArgs</param>
        /// <returns>Post spam rating</returns>
		public override int RatePost(Post Post, CSPostEventArgs Args)
		{
			// Verify that this spam rule should process this post
            // Also, only apply to anonymous users
            if (IsExemptFromSpamRating(Post, Args) || (!Post.User.IsAnonymous))
            {
                return 0;
            }

			// Retrieve setting values
			int points = Globals.SafeInt(this.GetSettingValue("points"), defaultPoints);
			int threshold = Globals.SafeInt(this.GetSettingValue("threshold"), defaultThreshold);

            // Calc. spam value
			if (!Globals.IsNullorEmpty(Post.Body))
			{
				int counter = Post.Body.Length;

				// Score 1 additional point for each character behind the threshold
                if (counter < threshold)
                {
                    return (points + (threshold - counter));
                }
			}

			return 0;
        }

        #endregion
    }
}
